@@ -9,8 +9,8 @@
 #include "DiscoverBackendsFactory.h"
 #include "DiscoverDeclarativePlugin.h"
 #include "FeaturedModel.h"
+#include "LimitedRowCountProxyModel.h"
 #include "OdrsAppsModel.h"
-#include "PaginateModel.h"
 #include "UnityLauncher.h"
 #include <Transaction/TransactionModel.h>
 
@@ -20,6 +20,7 @@
 #include <QDBusConnection>
 #include <QDBusMessage>
 #include <QDBusPendingCall>
+#include <QDBusPendingReply>
 #include <QDesktopServices>
 #include <QFile>
 #include <QGuiApplication>
@@ -28,7 +29,6 @@
 #include <QQmlContext>
 #include <QQmlEngine>
 #include <QSessionManager>
-#include <QSortFilterProxyModel>
 #include <QTimer>
 #include <QtQuick/QQuickItem>
 #include <qqml.h>
@@ -39,6 +39,7 @@
 #include <KConfigGroup>
 #include <KCrash>
 #include <KLocalizedContext>
+#include <KLocalizedQmlContext>
 #include <KLocalizedString>
 #include <KOSRelease>
 #include <KSharedConfig>
@@ -65,8 +66,11 @@
 #include "plasmauserfeedback.h"
 #endif
 #include "PowerManagementInterface.h"
+#include "RefreshNotifier.h"
 #include "discoversettings.h"
 
+using namespace Qt::StringLiterals;
+
 class CachedNetworkAccessManagerFactory : public QQmlNetworkAccessManagerFactory
 {
     virtual QNetworkAccessManager *create(QObject *parent) override
@@ -75,59 +79,54 @@ class CachedNetworkAccessManagerFactory : public QQmlNetworkAccessManagerFactory
     }
 };
 
-class OurSortFilterProxyModel : public QSortFilterProxyModel, public QQmlParserStatus
+static void scheduleShutdownWithErrorCode()
 {
-    Q_OBJECT
-    Q_INTERFACES(QQmlParserStatus)
-public:
-    void classBegin() override
-    {
-    }
-    void componentComplete() override
-    {
-        if (dynamicSortFilter())
-            sort(0);
-    }
-};
+    QTimer::singleShot(0, QCoreApplication::instance(), []() {
+        QCoreApplication::instance()->exit(1);
+    });
+}
 
-DiscoverObject::DiscoverObject(CompactMode mode, const QVariantMap &initialProperties)
+DiscoverObject::DiscoverObject(const QVariantMap &initialProperties)
     : QObject()
     , m_engine(new QQmlApplicationEngine)
-    , m_mode(mode)
     , m_networkAccessManagerFactory(new CachedNetworkAccessManagerFactory)
 {
     setObjectName(QStringLiteral("DiscoverMain"));
-    m_engine->rootContext()->setContextObject(new KLocalizedContext(m_engine));
+    m_engine->rootContext()->setContextObject(new KLocalizedQmlContext(m_engine));
     auto factory = m_engine->networkAccessManagerFactory();
     m_engine->setNetworkAccessManagerFactory(nullptr);
     delete factory;
     m_engine->setNetworkAccessManagerFactory(m_networkAccessManagerFactory.data());
 
-    qmlRegisterType<UnityLauncher>("org.kde.discover.app", 1, 0, "UnityLauncher");
-    qmlRegisterType<PaginateModel>("org.kde.discover.app", 1, 0, "PaginateModel");
-    qmlRegisterType<FeaturedModel>("org.kde.discover.app", 1, 0, "FeaturedModel");
-    qmlRegisterType<OdrsAppsModel>("org.kde.discover.app", 1, 0, "OdrsAppsModel");
-    qmlRegisterType<PowerManagementInterface>("org.kde.discover.app", 1, 0, "PowerManagementInterface");
-    qmlRegisterType<OurSortFilterProxyModel>("org.kde.discover.app", 1, 0, "QSortFilterProxyModel");
+    new RefreshNotifier(this);
+
+    const auto uriApp = "org.kde.discover.app";
+
+    qmlRegisterType<UnityLauncher>(uriApp, 1, 0, "UnityLauncher");
+    qmlRegisterType<LimitedRowCountProxyModel>(uriApp, 1, 0, "LimitedRowCountProxyModel");
+    qmlRegisterType<FeaturedModel>(uriApp, 1, 0, "FeaturedModel");
+    qmlRegisterType<OdrsAppsModel>(uriApp, 1, 0, "OdrsAppsModel");
+    qmlRegisterType<PowerManagementInterface>(uriApp, 1, 0, "PowerManagementInterface");
 #ifdef WITH_FEEDBACK
-    qmlRegisterSingletonType<PlasmaUserFeedback>("org.kde.discover.app", 1, 0, "UserFeedbackSettings", [](QQmlEngine *, QJSEngine *) -> QObject * {
-        return new PlasmaUserFeedback(KSharedConfig::openConfig(QStringLiteral("PlasmaUserFeedback"), KConfig::NoGlobals));
+    qmlRegisterSingletonType<PlasmaUserFeedback>(uriApp, 1, 0, "UserFeedbackSettings", [](QQmlEngine *engine, QJSEngine *) -> QObject * {
+        auto r = new PlasmaUserFeedback(KSharedConfig::openConfig(QStringLiteral("PlasmaUserFeedback"), KConfig::NoGlobals));
+        r->setParent(engine);
+        return r;
     });
 #endif
-    qmlRegisterSingletonType<DiscoverSettings>("org.kde.discover.app", 1, 0, "DiscoverSettings", [](QQmlEngine *engine, QJSEngine *) -> QObject * {
+    qmlRegisterSingletonType<DiscoverSettings>(uriApp, 1, 0, "DiscoverSettings", [](QQmlEngine *engine, QJSEngine *) -> QObject * {
         auto r = new DiscoverSettings;
         r->setParent(engine);
         connect(r, &DiscoverSettings::installedPageSortingChanged, r, &DiscoverSettings::save);
         connect(r, &DiscoverSettings::appsListPageSortingChanged, r, &DiscoverSettings::save);
         return r;
     });
-    qmlRegisterAnonymousType<QQuickView>("org.kde.discover.app", 1);
+    qmlRegisterAnonymousType<QQuickView>(uriApp, 1);
 
-    qmlRegisterAnonymousType<KAboutData>("org.kde.discover.app", 1);
-    qmlRegisterAnonymousType<KAboutLicense>("org.kde.discover.app", 1);
-    qmlRegisterAnonymousType<KAboutPerson>("org.kde.discover.app", 1);
-
-    qmlRegisterUncreatableType<DiscoverObject>("org.kde.discover.app", 1, 0, "DiscoverMainWindow", QStringLiteral("don't do that"));
+    qmlRegisterAnonymousType<KAboutData>(uriApp, 1);
+    qmlRegisterAnonymousType<KAboutLicense>(uriApp, 1);
+    qmlRegisterAnonymousType<KAboutPerson>(uriApp, 1);
+    qmlRegisterAnonymousType<DiscoverObject>(uriApp, 1);
 
     auto uri = "org.kde.discover";
     DiscoverDeclarativePlugin *plugin = new DiscoverDeclarativePlugin;
@@ -135,54 +134,83 @@ DiscoverObject::DiscoverObject(CompactMode mode, const QVariantMap &initialPrope
     plugin->initializeEngine(m_engine, uri);
     plugin->registerTypes(uri);
 
-    m_engine->setInitialProperties(initialProperties);
     m_engine->rootContext()->setContextProperty(QStringLiteral("app"), this);
-    m_engine->rootContext()->setContextProperty(QStringLiteral("discoverAboutData"), QVariant::fromValue(KAboutData::applicationData()));
 
-    connect(m_engine, &QQmlApplicationEngine::objectCreated, this, &DiscoverObject::integrateObject);
-    m_engine->load(QUrl(QStringLiteral("qrc:/qml/DiscoverWindow.qml")));
+    const auto discoverQmlUri = u"org.kde.discover.qml"_s;
+
+    auto navigation = m_engine->singletonInstance<QObject *>(discoverQmlUri, u"Navigation"_s);
+    if (!navigation) {
+        qCWarning(DISCOVER_LOG) << "Failed to import Navigation singleton";
+        scheduleShutdownWithErrorCode();
+        return;
+    }
+
+    {
+        QQmlComponent component(m_engine, discoverQmlUri, u"DiscoverWindow"_s);
+        auto object = component.beginCreate(m_engine->rootContext());
+        auto window = qobject_cast<QQuickWindow *>(object);
+        if (!object || !window) {
+            qCWarning(DISCOVER_LOG) << "Failed to create main window:" << component.errorString();
+            scheduleShutdownWithErrorCode();
+            component.completeCreate();
+            if (object) {
+                delete object;
+            }
+            return;
+        }
+        navigation->setProperty("window", QVariant::fromValue(object));
+        component.setInitialProperties(object, initialProperties);
+        component.completeCreate();
+        initMainWindow(window);
+    }
 
-    connect(QCoreApplication::instance(), &QCoreApplication::aboutToQuit, this, [this]() {
-        const auto objs = m_engine->rootObjects();
-        for (auto o : objs)
-            delete o;
-    });
     auto action = new OneTimeAction(
         [this]() {
             bool found = DiscoverBackendsFactory::hasRequestedBackends();
+            bool hasPackageKit = false;
             const auto backends = ResourcesModel::global()->backends();
-            for (auto b : backends)
+            for (auto b : backends) {
                 found |= b->hasApplications();
+                hasPackageKit = hasPackageKit || (b->name() == "packagekit-backend"_L1 && b->isValid());
+            }
 
+            const KOSRelease osRelease;
+            const QString distroName = osRelease.name();
+            const bool isArch = osRelease.id() == u"arch"_s || osRelease.idLike().contains(u"arch"_s);
             if (!found) {
-                QString errorText = i18n(
-                    "Discover currently cannot be used to install any apps or "
-                    "perform system updates because none of its app backends are "
-                    "available.");
-                QString errorExplanation = xi18nc("@info",
-                    "You can install some on the Settings page, under the "
-                    "<interface>Missing Backends</interface> section.<nl/><nl/>"
-                    "Also please consider reporting this as a packaging issue to "
-                    "your distribution.");
+                QString errorText =
+                    i18nc("@title %1 is the distro name", "%1 is not configured for installing apps through Discover—only app add-ons", distroName);
+                QString errorExplanation = xi18nc("@info:usagetip %1 is the distro name",
+                                                  "To use Discover for apps, install your preferred module on the <interface>Settings"
+                                                  "</interface> page, under <interface>Missing Backends</interface>.");
                 QString buttonIcon = QStringLiteral("tools-report-bug");
-                QString buttonText = i18n("Report This Issue");
-                QString buttonUrl = KOSRelease().bugReportUrl();
+                QString buttonText = i18nc("@action:button %1 is the distro name", "Report This Issue to %1", distroName);
+                QString buttonUrl = osRelease.bugReportUrl();
 
-                if (KOSRelease().name().contains(QStringLiteral("Arch Linux"))) {
-                    errorExplanation = xi18nc("@info",
-                        "You can use <command>pacman</command> to "
-                        "install the optional dependencies that are needed to "
-                        "enable the application backends.<nl/><nl/>Please note "
-                        "that Arch Linux developers recommend using "
-                        "<command>pacman</command> for managing software because "
-                        "the PackageKit backend is not well-integrated on Arch "
-                        "Linux.");
-                    buttonIcon = QStringLiteral("help-about");
-                    buttonText = i18n("Learn More");
-                    buttonUrl = KOSRelease().supportUrl();
+                if (isArch) {
+                    errorExplanation = xi18nc("@info:usagetip %1 is the distro name; in this case it always contains 'Arch Linux'",
+                                              "To use Discover for apps, install"
+                                              " <link url='https://wiki.archlinux.org/title/Flatpak#Installation'>Flatpak</link>"
+                                              " using the <command>pacman</command> package manager.<nl/><nl/>"
+                                              " Review <link url='https://archlinux.org/packages/extra/x86_64/discover/'>%1's packaging for Discover</link>",
+                                              distroName);
+                    buttonIcon = QString();
+                    buttonText = QString();
+                    buttonUrl = QString();
                 }
 
                 Q_EMIT openErrorPage(errorText, errorExplanation, buttonText, buttonIcon, buttonUrl);
+            } else if (hasPackageKit && isArch) {
+                const QString errorText = xi18nc("@info:usagetip %1 is the distro name",
+                                                 "Support for managing packages from %1 is incomplete; you may experience any number of problems."
+                                                 " Do not report bugs to KDE. It is highly recommended to uninstall the"
+                                                 " <resource>packagekit-qt6</resource> package and use Discover only to manage Flatpaks, Snaps,"
+                                                 " and Add-Ons."
+                                                 "<para>%1 maintainers recommended instead using the <command>pacman</command> command-line tool"
+                                                 " to manage packages.</para>",
+                                                 distroName);
+                m_homePageMessage = std::make_unique<InlineMessage>(InlineMessage::Warning, QString(), errorText);
+                Q_EMIT homeMessageChanged();
             }
         },
         this);
@@ -207,7 +235,7 @@ bool DiscoverObject::isRoot()
 QStringList DiscoverObject::modes() const
 {
     QStringList ret;
-    QObject *obj = rootObject();
+    QObject *obj = mainWindow();
     for (int i = obj->metaObject()->propertyOffset(); i < obj->metaObject()->propertyCount(); i++) {
         QMetaProperty p = obj->metaObject()->property(i);
         QByteArray compName = p.name();
@@ -220,8 +248,7 @@ QStringList DiscoverObject::modes() const
 
 void DiscoverObject::openMode(const QString &_mode)
 {
-    QObject *obj = rootObject();
-    if (!obj) {
+    if (!m_mainWindow) {
         qCWarning(DISCOVER_LOG) << "could not get the main object";
         return;
     }
@@ -233,11 +260,12 @@ void DiscoverObject::openMode(const QString &_mode)
     mode[0] = mode[0].toUpper();
 
     const QByteArray propertyName = "top" + mode.toLatin1() + "Comp";
-    const QVariant modeComp = obj->property(propertyName.constData());
-    if (!modeComp.isValid())
+    const QVariant modeComp = m_mainWindow->property(propertyName.constData());
+    if (!modeComp.isValid()) {
         qCWarning(DISCOVER_LOG) << "couldn't open mode" << _mode;
-    else
-        obj->setProperty("currentTopLevel", modeComp);
+    } else {
+        m_mainWindow->setProperty("currentTopLevel", modeComp);
+    }
 }
 
 void DiscoverObject::openMimeType(const QString &mime)
@@ -247,13 +275,9 @@ void DiscoverObject::openMimeType(const QString &mime)
 
 void DiscoverObject::showLoadingPage()
 {
-    QObject *obj = rootObject();
-    if (!obj) {
-        qCWarning(DISCOVER_LOG) << "could not get the main object";
-        return;
+    if (m_mainWindow) {
+        m_mainWindow->setProperty("currentTopLevel", QStringLiteral(DISCOVER_BASE_URL "/LoadingPage.qml"));
     }
-
-    obj->setProperty("currentTopLevel", QStringLiteral("qrc:/qml/LoadingPage.qml"));
 }
 
 void DiscoverObject::openCategory(const QString &category)
@@ -291,9 +315,9 @@ void DiscoverObject::openLocalPackage(const QUrl &localfile)
             AbstractResourcesBackend::Filters f;
             f.resourceUrl = localfile;
             auto stream = new StoredResultsStream({ResourcesModel::global()->search(f)});
-            connect(stream, &StoredResultsStream::finishedResources, this, [this, localfile](const QVector<AbstractResource *> &res) {
+            connect(stream, &StoredResultsStream::finishedResources, this, [this, localfile](const QVector<StreamResult> &res) {
                 if (res.count() == 1) {
-                    Q_EMIT openApplicationInternal(res.first());
+                    Q_EMIT openApplicationInternal(res.first().resource);
                 } else {
                     QMimeDatabase db;
                     auto mime = db.mimeTypeForUrl(localfile);
@@ -330,7 +354,7 @@ void DiscoverObject::openApplication(const QUrl &url)
             AbstractResourcesBackend::Filters f;
             f.resourceUrl = url;
             auto stream = new StoredResultsStream({ResourcesModel::global()->search(f)});
-            connect(stream, &StoredResultsStream::finishedResources, this, [this, url](const QVector<AbstractResource *> &res) {
+            connect(stream, &StoredResultsStream::finishedResources, this, [this, url](const QVector<StreamResult> &res) {
                 if (res.count() >= 1) {
                     QPointer<QTimer> timeout = new QTimer(this);
                     timeout->setSingleShot(true);
@@ -339,11 +363,11 @@ void DiscoverObject::openApplication(const QUrl &url)
 
                     auto openResourceOrWait = [this, res, timeout] {
                         auto idx = kIndexOf(res, [](auto res) {
-                            return res->isInstalled();
+                            return res.resource->isInstalled();
                         });
                         if (idx < 0) {
                             bool oneBroken = kContains(res, [](auto res) {
-                                return res->state() == AbstractResource::Broken;
+                                return res.resource->state() == AbstractResource::Broken;
                             });
                             if (oneBroken && timeout) {
                                 return false;
@@ -351,15 +375,15 @@ void DiscoverObject::openApplication(const QUrl &url)
 
                             idx = 0;
                         }
-                        Q_EMIT openApplicationInternal(res[idx]);
+                        Q_EMIT openApplicationInternal(res[idx].resource);
                         return true;
                     };
 
                     if (!openResourceOrWait()) {
                         auto f = new OneTimeAction(0, openResourceOrWait, this);
                         for (auto r : res) {
-                            if (r->state() == AbstractResource::Broken) {
-                                connect(r, &AbstractResource::stateChanged, f, &OneTimeAction::trigger);
+                            if (r.resource->state() == AbstractResource::Broken) {
+                                connect(r.resource, &AbstractResource::stateChanged, f, &OneTimeAction::trigger);
                             }
                         }
                         timeout->start();
@@ -371,11 +395,13 @@ void DiscoverObject::openApplication(const QUrl &url)
                     openApplication(QUrl(QStringLiteral("appstream://org.kde.discover.snap")));
                     showError(i18n("Please make sure Snap support is installed"));
                 } else {
-                    const QString errorText = i18n("Could not open %1 because it "
-                    "was not found in any available software repositories.",
-                    url.toDisplayString());
-                    const QString errorExplanation = i18n("Please report this "
-                    "issue to the packagers of your distribution.");
+                    const QString errorText = i18n(
+                        "Could not open %1 because it "
+                        "was not found in any available software repositories.",
+                        url.toDisplayString());
+                    const QString errorExplanation = i18n(
+                        "Please report this "
+                        "issue to the packagers of your distribution.");
                     QString buttonIcon = QStringLiteral("tools-report-bug");
                     QString buttonText = i18n("Report This Issue");
                     QString buttonUrl = KOSRelease().bugReportUrl();
@@ -438,35 +464,37 @@ bool DiscoverObject::quitWhenIdle()
 
         m_sni = new KStatusNotifierItem(this);
         m_sni->setStatus(KStatusNotifierItem::Active);
-        m_sni->setIconByName("plasmadiscover");
+        m_sni->setIconByName(QStringLiteral("plasmadiscover"));
         m_sni->setTitle(i18n("Discover"));
-        m_sni->setToolTip("process-working-symbolic", i18n("Discover"), i18n("Discover was closed before certain tasks were done, waiting for it to finish."));
+        m_sni->setToolTip(QStringLiteral("process-working-symbolic"),
+                          i18n("Discover"),
+                          i18n("Discover was closed before certain tasks were done, waiting for it to finish."));
         m_sni->setStandardActionsEnabled(false);
 
         connect(TransactionModel::global(), &TransactionModel::countChanged, this, &DiscoverObject::reconsiderQuit);
         connect(m_sni, &KStatusNotifierItem::activateRequested, this, &DiscoverObject::restore);
 
         auto job = new TransactionsJob;
-        job->setParent(this);
+        job->setParent(m_sni);
         tracker->registerJob(job);
         job->start();
         connect(m_sni, &KStatusNotifierItem::activateRequested, job, &TransactionsJob::cancel);
 
-        rootObject()->hide();
+        m_mainWindow->hide();
     }
     return false;
 }
 
 void DiscoverObject::restore()
 {
-    if (!m_sni) {
+    if (!m_sni || !m_mainWindow) {
         return;
     }
 
     disconnect(TransactionModel::global(), &TransactionModel::countChanged, this, &DiscoverObject::reconsiderQuit);
     disconnect(m_sni, &KStatusNotifierItem::activateRequested, this, &DiscoverObject::restore);
 
-    rootObject()->show();
+    m_mainWindow->show();
     m_sni->deleteLater();
     m_sni = nullptr;
 }
@@ -482,34 +510,23 @@ void DiscoverObject::reconsiderQuit()
     QTimer::singleShot(20, qGuiApp, &QCoreApplication::quit);
 }
 
-void DiscoverObject::integrateObject(QObject *object)
+void DiscoverObject::initMainWindow(QQuickWindow *mainWindow)
 {
-    if (!object) {
-        qCWarning(DISCOVER_LOG) << "Errors when loading the GUI";
-        QTimer::singleShot(0, QCoreApplication::instance(), []() {
-            QCoreApplication::instance()->exit(1);
-        });
-        return;
-    }
+    Q_ASSERT(mainWindow);
 
-    Q_ASSERT(object == rootObject());
+    m_mainWindow = std::unique_ptr<QQuickWindow>(mainWindow);
 
-    KConfigGroup window(KSharedConfig::openConfig(), "Window");
-    if (window.hasKey("geometry"))
-        rootObject()->setGeometry(window.readEntry("geometry", QRect()));
-    if (window.hasKey("visibility")) {
-        QWindow::Visibility visibility(QWindow::Visibility(window.readEntry<int>("visibility", QWindow::Windowed)));
-        rootObject()->setVisibility(qMax(visibility, QQuickView::AutomaticVisibility));
-    }
-    connect(rootObject(), &QQuickView::sceneGraphError, this, [](QQuickWindow::SceneGraphError /*error*/, const QString &message) {
+    connect(m_mainWindow.get(), &QQuickView::sceneGraphError, this, [](QQuickWindow::SceneGraphError /*error*/, const QString &message) {
         KCrash::setErrorMessage(message);
         qFatal("%s", qPrintable(message));
     });
 
-    object->installEventFilter(this);
-    connect(object, &QObject::destroyed, qGuiApp, &QCoreApplication::quit);
+    m_mainWindow->installEventFilter(this);
+
+    connect(QCoreApplication::instance(), &QCoreApplication::aboutToQuit, this, [this]() {
+        m_mainWindow.reset();
+    });
 
-    object->setParent(m_engine);
     connect(qGuiApp, &QGuiApplication::commitDataRequest, this, [this](QSessionManager &sessionManager) {
         if (!quitWhenIdle()) {
             sessionManager.cancel();
@@ -519,20 +536,18 @@ void DiscoverObject::integrateObject(QObject *object)
 
 bool DiscoverObject::eventFilter(QObject *object, QEvent *event)
 {
-    if (object != rootObject())
+    if (object != m_mainWindow.get()) {
         return false;
+    }
 
     if (event->type() == QEvent::Close) {
         if (!quitWhenIdle()) {
             return true;
         }
-
-        KConfigGroup window(KSharedConfig::openConfig(), "Window");
-        window.writeEntry("geometry", rootObject()->geometry());
-        window.writeEntry<int>("visibility", rootObject()->visibility());
-        // } else if (event->type() == QEvent::ShortcutOverride) {
-        //     qCWarning(DISCOVER_LOG) << "Action conflict" << event;
     }
+    // } else if (event->type() == QEvent::ShortcutOverride) {
+    //     qCWarning(DISCOVER_LOG) << "Action conflict" << event;
+    // }
     return false;
 }
 
@@ -548,18 +563,10 @@ void DiscoverObject::switchApplicationLanguage()
     // langDialog->show();
 }
 
-void DiscoverObject::setCompactMode(DiscoverObject::CompactMode mode)
-{
-    if (m_mode != mode) {
-        m_mode = mode;
-        Q_EMIT compactModeChanged(m_mode);
-    }
-}
-
 class DiscoverTestExecutor : public QObject
 {
 public:
-    DiscoverTestExecutor(QObject *rootObject, QQmlEngine *engine, const QUrl &url)
+    DiscoverTestExecutor(QQuickWindow *mainWindow, QQmlEngine *engine, const QUrl &url)
         : QObject(engine)
     {
         connect(engine, &QQmlEngine::quit, this, &DiscoverTestExecutor::finish, Qt::QueuedConnection);
@@ -572,7 +579,7 @@ public:
             Q_ASSERT(false);
         }
 
-        m_testObject->setProperty("appRoot", QVariant::fromValue<QObject *>(rootObject));
+        m_testObject->setProperty("appRoot", QVariant::fromValue(mainWindow));
         connect(engine, &QQmlEngine::warnings, this, &DiscoverTestExecutor::processWarnings);
     }
 
@@ -603,12 +610,12 @@ private:
 
 void DiscoverObject::loadTest(const QUrl &url)
 {
-    new DiscoverTestExecutor(rootObject(), engine(), url);
+    new DiscoverTestExecutor(m_mainWindow.get(), engine(), url);
 }
 
-QQuickWindow *DiscoverObject::rootObject() const
+QQuickWindow *DiscoverObject::mainWindow() const
 {
-    return qobject_cast<QQuickWindow *>(m_engine->rootObjects().at(0));
+    return m_mainWindow.get();
 }
 
 void DiscoverObject::showError(const QString &msg)
@@ -624,7 +631,21 @@ void DiscoverObject::copyTextToClipboard(const QString text)
     clipboard->setText(text);
 }
 
-void DiscoverObject::reboot()
+void DiscoverObject::setAboutToPowerOff()
+{
+    for (auto backend : ResourcesModel::global()->backends()) {
+        backend->aboutTo(AbstractResourcesBackend::PowerOff);
+    }
+}
+
+void DiscoverObject::setAboutToReboot()
+{
+    for (auto backend : ResourcesModel::global()->backends()) {
+        backend->aboutTo(AbstractResourcesBackend::Reboot);
+    }
+}
+
+void DiscoverObject::promptReboot()
 {
     auto method = QDBusMessage::createMethodCall(QStringLiteral("org.kde.LogoutPrompt"),
                                                  QStringLiteral("/LogoutPrompt"),
@@ -635,6 +656,7 @@ void DiscoverObject::reboot()
 
 void DiscoverObject::rebootNow()
 {
+    setAboutToReboot();
     auto method = QDBusMessage::createMethodCall(QStringLiteral("org.freedesktop.login1"),
                                                  QStringLiteral("/org/freedesktop/login1"),
                                                  QStringLiteral("org.freedesktop.login1.Manager"),
@@ -643,15 +665,34 @@ void DiscoverObject::rebootNow()
     QDBusConnection::systemBus().asyncCall(method);
 }
 
-QRect DiscoverObject::initialGeometry() const
+void DiscoverObject::shutdownNow()
 {
-    KConfigGroup window(KSharedConfig::openConfig(), "Window");
-    return window.readEntry("geometry", QRect());
+    bool shouldRebootFirst = false;
+    for (auto backend : ResourcesModel::global()->backends()) {
+        if (backend->needsRebootForPowerOffAction()) {
+            shouldRebootFirst = true;
+            break;
+        }
+    }
+    setAboutToPowerOff();
+
+    auto method = QDBusMessage::createMethodCall(QStringLiteral("org.freedesktop.login1"),
+                                                 QStringLiteral("/org/freedesktop/login1"),
+                                                 QStringLiteral("org.freedesktop.login1.Manager"),
+                                                 shouldRebootFirst ? QStringLiteral("Reboot") : QStringLiteral("PowerOff"));
+    method.setArguments({true /*interactive*/});
+    QDBusConnection::systemBus().asyncCall(method);
 }
 
 QString DiscoverObject::describeSources() const
 {
-    return rootObject()->property("describeSources").toString();
+    return mainWindow()->property("describeSources").toString();
+}
+
+InlineMessage *DiscoverObject::homePageMessage() const
+{
+    return m_homePageMessage.get();
 }
 
 #include "DiscoverObject.moc"
+#include "moc_DiscoverObject.cpp"
