@@ -4,8 +4,7 @@
  *   SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
-#ifndef DISCOVEROBJECT_H
-#define DISCOVEROBJECT_H
+#pragma once
 
 #include <QUrl>
 
@@ -13,36 +12,28 @@
 
 class AbstractResource;
 class Category;
+class KStatusNotifierItem;
 class QWindow;
 class QQmlApplicationEngine;
 class CachedNetworkAccessManagerFactory;
+class TransactionsJob;
+class InlineMessage;
+
+#define DISCOVER_BASE_URL "qrc:/qt/qml/org/kde/discover/qml"
 
 class DiscoverObject : public QObject
 {
     Q_OBJECT
-    Q_PROPERTY(CompactMode compactMode READ compactMode WRITE setCompactMode NOTIFY compactModeChanged)
     Q_PROPERTY(bool isRoot READ isRoot CONSTANT)
-    Q_PROPERTY(QRect initialGeometry READ initialGeometry CONSTANT)
+    Q_PROPERTY(QQuickWindow *mainWindow READ mainWindow CONSTANT)
+    Q_PROPERTY(InlineMessage *homePageMessage READ homePageMessage NOTIFY homeMessageChanged)
 
 public:
-    enum CompactMode {
-        Auto,
-        Compact,
-        Full,
-    };
-    Q_ENUM(CompactMode)
-
-    explicit DiscoverObject(CompactMode mode, const QVariantMap &initialProperties);
+    explicit DiscoverObject(const QVariantMap &initialProperties);
     ~DiscoverObject() override;
 
     QStringList modes() const;
 
-    CompactMode compactMode() const
-    {
-        return m_mode;
-    }
-    void setCompactMode(CompactMode mode);
-
     bool eventFilter(QObject *object, QEvent *event) override;
 
     Q_SCRIPTABLE static QString iconName(const QIcon &icon);
@@ -50,12 +41,13 @@ public:
     void loadTest(const QUrl &url);
 
     static bool isRoot();
-    QQuickWindow *rootObject() const;
-    void showPassiveNotification(const QString &msg);
+    QQuickWindow *mainWindow() const;
+    void showError(const QString &msg);
     Q_INVOKABLE void copyTextToClipboard(const QString text);
-    QRect initialGeometry() const;
 
     QString describeSources() const;
+    Q_SCRIPTABLE void restore();
+    [[nodiscard]] InlineMessage *homePageMessage() const;
 
 public Q_SLOTS:
     void openApplication(const QUrl &app);
@@ -64,8 +56,9 @@ public Q_SLOTS:
     void openMode(const QString &mode);
     void openLocalPackage(const QUrl &localfile);
 
-    void reboot();
+    void promptReboot();
     void rebootNow();
+    void shutdownNow();
 
 private Q_SLOTS:
     void switchApplicationLanguage();
@@ -76,23 +69,27 @@ Q_SIGNALS:
     void listMimeInternal(const QString &mime);
     void listCategoryInternal(Category *cat);
 
-    void compactModeChanged(DiscoverObject::CompactMode compactMode);
-    void preventedClose();
     void unableToFind(const QString &resid);
-    void openErrorPage(const QString &errorMessage);
+    void
+    openErrorPage(const QString &errorMessage, const QString &errorExplanation, const QString &buttonText, const QString &buttonIcon, const QString &buttonURL);
+    void homeMessageChanged();
 
 private:
     void showLoadingPage();
-    void integrateObject(QObject *object);
+    void initMainWindow(QQuickWindow *mainWindow);
+    bool quitWhenIdle();
+    void reconsiderQuit();
+    void setAboutToReboot();
+    void setAboutToPowerOff();
     QQmlApplicationEngine *engine() const
     {
         return m_engine;
     }
 
     QQmlApplicationEngine *const m_engine;
+    std::unique_ptr<QQuickWindow> m_mainWindow;
 
-    CompactMode m_mode;
     QScopedPointer<CachedNetworkAccessManagerFactory> m_networkAccessManagerFactory;
+    KStatusNotifierItem *m_sni = nullptr;
+    std::unique_ptr<InlineMessage> m_homePageMessage;
 };
-
-#endif // DISCOVEROBJECT_H
