@@ -48,11 +48,11 @@ QHash<int, QByteArray> UpdateModel::roleNames() const
     ret.insert(Qt::CheckStateRole, "checked");
     ret.insert(ResourceProgressRole, "resourceProgress");
     ret.insert(ResourceStateRole, "resourceState");
+    ret.insert(ResourceStateIsDoneRole, "resourceStateIsDone");
     ret.insert(ResourceRole, "resource");
     ret.insert(SizeRole, "size");
     ret.insert(SectionRole, "section");
     ret.insert(ChangelogRole, "changelog");
-    ret.insert(UpgradeTextRole, "upgradeText");
     ret.insert(ExtendedRole, "extended");
     return ret;
 }
@@ -60,13 +60,16 @@ QHash<int, QByteArray> UpdateModel::roleNames() const
 void UpdateModel::setBackend(ResourcesUpdatesModel *updates)
 {
     if (m_updates) {
-        disconnect(m_updates, nullptr, this, nullptr);
+        disconnect(m_updates, &ResourcesUpdatesModel::progressingChanged, this, &UpdateModel::activityChanged);
+        disconnect(m_updates, &ResourcesUpdatesModel::resourceProgressed, this, &UpdateModel::resourceHasProgressed);
     }
 
     m_updates = updates;
 
-    connect(m_updates, &ResourcesUpdatesModel::progressingChanged, this, &UpdateModel::activityChanged);
-    connect(m_updates, &ResourcesUpdatesModel::resourceProgressed, this, &UpdateModel::resourceHasProgressed);
+    if (m_updates) {
+        connect(m_updates, &ResourcesUpdatesModel::progressingChanged, this, &UpdateModel::activityChanged);
+        connect(m_updates, &ResourcesUpdatesModel::resourceProgressed, this, &UpdateModel::resourceHasProgressed);
+    }
 
     activityChanged();
 }
@@ -74,13 +77,14 @@ void UpdateModel::setBackend(ResourcesUpdatesModel *updates)
 void UpdateModel::resourceHasProgressed(AbstractResource *res, qreal progress, AbstractBackendUpdater::State state)
 {
     UpdateItem *item = itemFromResource(res);
-    if (!item)
+    if (!item) {
         return;
+    }
     item->setProgress(progress);
     item->setState(state);
 
     const QModelIndex idx = indexFromItem(item);
-    Q_EMIT dataChanged(idx, idx, {ResourceProgressRole, ResourceStateRole, SectionResourceProgressRole});
+    Q_EMIT dataChanged(idx, idx, {ResourceProgressRole, ResourceStateRole, ResourceStateIsDoneRole, SectionResourceProgressRole});
 }
 
 void UpdateModel::activityChanged()
@@ -90,11 +94,12 @@ void UpdateModel::activityChanged()
             m_updates->prepare();
             setResources(m_updates->toUpdate());
 
-            for (auto item : qAsConst(m_updateItems)) {
+            for (auto item : std::as_const(m_updateItems)) {
                 item->setProgress(0);
             }
-        } else
+        } else {
             setResources(m_updates->toUpdate());
+        }
     }
 }
 
@@ -121,18 +126,23 @@ QVariant UpdateModel::data(const QModelIndex &index, int role) const
         return item->progress();
     case ResourceStateRole:
         return item->state();
+    case ResourceStateIsDoneRole:
+        return item->state() == AbstractBackendUpdater::State::Done;
     case ChangelogRole:
         return item->changelog();
     case ExtendedRole:
         return item->isExtended();
     case SectionRole: {
         static const QString appUpdatesSection = i18nc("@item:inlistbox", "Applications");
+        static const QString applicationSupportSection = i18nc("@item:inlistbox", "Application Support");
         static const QString systemUpdateSection = i18nc("@item:inlistbox", "System Software");
         static const QString addonsSection = i18nc("@item:inlistbox", "Addons");
         switch (item->resource()->type()) {
         case AbstractResource::Application:
             return appUpdatesSection;
-        case AbstractResource::Technical:
+        case AbstractResource::ApplicationSupport:
+            return applicationSupportSection;
+        case AbstractResource::System:
             return systemUpdateSection;
         case AbstractResource::Addon:
             return addonsSection;
@@ -150,16 +160,18 @@ QVariant UpdateModel::data(const QModelIndex &index, int role) const
 
 void UpdateModel::checkResources(const QList<AbstractResource *> &resource, bool checked)
 {
-    if (checked)
+    if (checked) {
         m_updates->addResources(resource);
-    else
+    } else {
         m_updates->removeResources(resource);
+    }
 }
 
 Qt::ItemFlags UpdateModel::flags(const QModelIndex &index) const
 {
-    if (!index.isValid())
+    if (!index.isValid()) {
         return Qt::NoItemFlags;
+    }
 
     return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
 }
@@ -200,8 +212,9 @@ void UpdateModel::fetchUpdateDetails(int row)
 {
     UpdateItem *item = itemFromIndex(index(row, 0));
     Q_ASSERT(item);
-    if (!item)
+    if (!item) {
         return;
+    }
 
     item->app()->fetchUpdateDetails();
 }
@@ -211,8 +224,9 @@ void UpdateModel::integrateChangelog(const QString &changelog)
     auto app = qobject_cast<AbstractResource *>(sender());
     Q_ASSERT(app);
     auto item = itemFromResource(app);
-    if (!item)
+    if (!item) {
         return;
+    }
 
     item->setChangelog(changelog);
 
@@ -232,31 +246,37 @@ void UpdateModel::setResources(const QList<AbstractResource *> &resources)
     qDeleteAll(m_updateItems);
     m_updateItems.clear();
 
-    QVector<UpdateItem *> appItems, systemItems, addonItems;
-    for (AbstractResource *res : resources) {
-        connect(res, &AbstractResource::changelogFetched, this, &UpdateModel::integrateChangelog, Qt::UniqueConnection);
+    QVector<UpdateItem *> appItems, appSupportItems, systemItems, addonItems;
+    for (AbstractResource *resource : resources) {
+        connect(resource, &AbstractResource::changelogFetched, this, &UpdateModel::integrateChangelog, Qt::UniqueConnection);
 
-        UpdateItem *updateItem = new UpdateItem(res);
+        UpdateItem *updateItem = new UpdateItem(resource);
 
-        switch (res->type()) {
-        case AbstractResource::Technical:
-            systemItems += updateItem;
-            break;
+        switch (resource->type()) {
         case AbstractResource::Application:
             appItems += updateItem;
             break;
+        case AbstractResource::ApplicationSupport:
+            appSupportItems += updateItem;
+            break;
+        case AbstractResource::System:
+            systemItems += updateItem;
+            break;
         case AbstractResource::Addon:
             addonItems += updateItem;
             break;
         }
     }
-    const auto sortUpdateItems = [](UpdateItem *a, UpdateItem *b) {
-        return a->name() < b->name();
+
+    QCollator collator;
+    const auto sortUpdateItems = [&collator](UpdateItem *a, UpdateItem *b) -> bool {
+        return collator(a->name(), b->name());
     };
     std::sort(appItems.begin(), appItems.end(), sortUpdateItems);
+    std::sort(appSupportItems.begin(), appSupportItems.end(), sortUpdateItems);
     std::sort(systemItems.begin(), systemItems.end(), sortUpdateItems);
     std::sort(addonItems.begin(), addonItems.end(), sortUpdateItems);
-    m_updateItems = (QVector<UpdateItem *>() << appItems << addonItems << systemItems);
+    m_updateItems = (QVector<UpdateItem *>() << appItems << addonItems << appSupportItems << systemItems);
     endResetModel();
 
     Q_EMIT hasUpdatesChanged(!resources.isEmpty());
@@ -277,7 +297,7 @@ int UpdateModel::toUpdateCount() const
 {
     int ret = 0;
     QSet<QString> packages;
-    for (UpdateItem *item : qAsConst(m_updateItems)) {
+    for (UpdateItem *item : std::as_const(m_updateItems)) {
         const auto packageName = item->resource()->packageName();
         if (packages.contains(packageName)) {
             continue;
@@ -292,7 +312,7 @@ int UpdateModel::totalUpdatesCount() const
 {
     int ret = 0;
     QSet<QString> packages;
-    for (UpdateItem *item : qAsConst(m_updateItems)) {
+    for (UpdateItem *item : std::as_const(m_updateItems)) {
         const auto packageName = item->resource()->packageName();
         if (packages.contains(packageName)) {
             continue;
@@ -305,9 +325,10 @@ int UpdateModel::totalUpdatesCount() const
 
 UpdateItem *UpdateModel::itemFromResource(AbstractResource *res)
 {
-    for (UpdateItem *item : qAsConst(m_updateItems)) {
-        if (item->app() == res)
+    for (UpdateItem *item : std::as_const(m_updateItems)) {
+        if (item->app() == res) {
             return item;
+        }
     }
     return nullptr;
 }
@@ -336,13 +357,17 @@ UpdateItem *UpdateModel::itemFromIndex(const QModelIndex &index) const
 void UpdateModel::resourceDataChanged(AbstractResource *res, const QVector<QByteArray> &properties)
 {
     auto item = itemFromResource(res);
-    if (!item)
+    if (!item) {
         return;
+    }
 
     const auto index = indexFromItem(item);
-    if (properties.contains("state"))
-        Q_EMIT dataChanged(index, index, {SizeRole, UpgradeTextRole});
-    else if (properties.contains("size")) {
+    if (properties.contains("icon")) {
+        Q_EMIT dataChanged(index, index, {Qt::DecorationRole});
+    }
+    if (properties.contains("state")) {
+        Q_EMIT dataChanged(index, index, {SizeRole});
+    } else if (properties.contains("size")) {
         Q_EMIT dataChanged(index, index, {SizeRole});
         m_updateSizeTimer->start();
     }
