@@ -34,7 +34,7 @@ void DummyTransaction::iterateTransaction()
         return;
 
     if (progress() < 100) {
-        setProgress(qBound(0, progress() + (KRandom::random() % 5), 100));
+        setProgress(qBound(0, progress() + QRandomGenerator::global()->bounded(5), 100));
         QTimer::singleShot(/*KRandom::random()%*/ 100, this, &DummyTransaction::iterateTransaction);
     } else if (status() == DownloadingStatus) {
         setStatus(CommittingStatus);
@@ -64,7 +64,7 @@ void DummyTransaction::cancel()
 
 void DummyTransaction::finishTransaction()
 {
-    AbstractResource::State newState;
+    AbstractResource::State newState = AbstractResource::State::Broken;
     switch (role()) {
     case InstallRole:
     case ChangeAddonsRole:
@@ -79,3 +79,5 @@ void DummyTransaction::finishTransaction()
     setStatus(DoneStatus);
     deleteLater();
 }
+
+#include "moc_DummyTransaction.cpp"
