@@ -33,10 +33,10 @@ public:
 
     void setProgressing(bool progressing);
 
-    void removeResources(const QList<AbstractResource *> &apps) override;
-    void addResources(const QList<AbstractResource *> &apps) override;
+    void removeResources(const QList<AbstractResource *> &resources) override;
+    void addResources(const QList<AbstractResource *> &resources) override;
     QList<AbstractResource *> toUpdate() const override;
-    bool isMarked(AbstractResource *res) const override;
+    bool isMarked(AbstractResource *resource) const override;
     QDateTime lastUpdate() const override;
     bool isCancelable() const override;
     bool isProgressing() const override;
@@ -46,6 +46,10 @@ public:
 
     void proceed() override;
     void setOfflineUpdates(bool use) override;
+    void setDistroUpgrade(const AppStream::Release &release);
+    void clearDistroUpgrade();
+    bool isDistroUpgrade() const;
+    void setOfflineUpdateAction(PackageKit::Offline::Action action);
 
 public Q_SLOTS:
     /// must be implemented if ever isCancelable is true
@@ -103,6 +107,7 @@ private:
     QDateTime m_lastUpdate;
     QMap<PackageKit::Transaction::Info, QStringList> m_packagesModified;
     QVector<std::function<PackageKit::Transaction *()>> m_proceedFunctions;
+    PackageKit::Offline::Action m_offlineUpdateAction = PackageKit::Offline::Action::ActionReboot;
 
     SystemUpgrade *m_upgrade = nullptr;
 };
