@@ -15,7 +15,6 @@
 #include <QProcess>
 #include <QString>
 #include <ReviewsBackend/AbstractReviewsBackend.h>
-#include <ReviewsBackend/Rating.h>
 
 AbstractResource::AbstractResource(AbstractResourcesBackend *parent)
     : QObject(parent)
@@ -52,12 +51,12 @@ QUrl AbstractResource::contributeURL()
     return {};
 }
 
-void AbstractResource::addMetadata(const QString &key, const QJsonValue &value)
+void AbstractResource::addMetadata(const QLatin1StringView &key, const QJsonValue &value)
 {
     m_metadata.insert(key, value);
 }
 
-QJsonValue AbstractResource::getMetadata(const QString &key)
+QJsonValue AbstractResource::getMetadata(const QLatin1StringView &key)
 {
     return m_metadata.value(key);
 }
@@ -87,11 +86,6 @@ AbstractResourcesBackend *AbstractResource::backend() const
     return static_cast<AbstractResourcesBackend *>(parent());
 }
 
-QObject *AbstractResource::backendObject() const
-{
-    return parent();
-}
-
 QString AbstractResource::status()
 {
     switch (state()) {
@@ -115,21 +109,18 @@ QString AbstractResource::sizeDescription()
 QCollatorSortKey AbstractResource::nameSortKey()
 {
     if (!m_collatorKey) {
-        m_collatorKey.reset(new QCollatorSortKey(QCollator().sortKey(name())));
+        static QCollator collator;
+        m_collatorKey.reset(new QCollatorSortKey(collator.sortKey(name())));
     }
     return *m_collatorKey;
 }
 
-Rating *AbstractResource::rating() const
+Rating AbstractResource::rating() const
 {
-    AbstractReviewsBackend *ratings = backend()->reviewsBackend();
-    return ratings ? ratings->ratingForApplication(const_cast<AbstractResource *>(this)) : nullptr;
-}
-
-QVariant AbstractResource::ratingVariant() const
-{
-    auto instance = rating();
-    return instance ? QVariant::fromValue<Rating>(*instance) : QVariant();
+    if (auto reviewsBackend = backend()->reviewsBackend()) {
+        return reviewsBackend->ratingForApplication(const_cast<AbstractResource *>(this));
+    }
+    return {};
 }
 
 QStringList AbstractResource::extends() const
@@ -144,54 +135,65 @@ QString AbstractResource::appstreamId() const
 
 void AbstractResource::reportNewState()
 {
-    if (backend()->isFetching())
+    if (backend()->isFetching()) {
         return;
+    }
 
-    static const QVector<QByteArray> ns = {"state", "status", "canUpgrade", "size", "sizeDescription", "installedVersion", "availableVersion"};
-    Q_EMIT backend()->resourcesChanged(this, ns);
+    static const QVector<QByteArray> properties = {
+        "state",
+        "status",
+        "canUpgrade",
+        "size",
+        "sizeDescription",
+        "installedVersion",
+        "availableVersion",
+    };
+    Q_EMIT backend()->resourcesChanged(this, properties);
 }
 
-static bool shouldFilter(AbstractResource *res, const CategoryFilter &filter)
+static bool shouldFilter(AbstractResource *resource, const CategoryFilter &filter)
 {
     bool ret = true;
     switch (filter.type) {
     case CategoryFilter::CategoryNameFilter:
-        ret = res->categories().contains(std::get<QString>(filter.value));
+        ret = resource->hasCategory(std::get<QString>(filter.value));
         break;
     case CategoryFilter::PkgSectionFilter:
-        ret = res->section() == std::get<QString>(filter.value);
+        ret = resource->section() == std::get<QString>(filter.value);
         break;
     case CategoryFilter::PkgWildcardFilter: {
         QString wildcard = std::get<QString>(filter.value);
         wildcard.remove(QLatin1Char('*'));
-        ret = res->packageName().contains(wildcard);
-    } break;
+        ret = resource->packageName().contains(wildcard);
+        break;
+    }
     case CategoryFilter::AppstreamIdWildcardFilter: {
         QString wildcard = std::get<QString>(filter.value);
         wildcard.remove(QLatin1Char('*'));
-        ret = res->appstreamId().contains(wildcard);
-    } break;
+        ret = resource->appstreamId().contains(wildcard);
+        break;
+    }
     case CategoryFilter::PkgNameFilter: // Only useful in the not filters
-        ret = res->packageName() == std::get<QString>(filter.value);
+        ret = resource->packageName() == std::get<QString>(filter.value);
         break;
     case CategoryFilter::AndFilter: {
         const auto filters = std::get<QVector<CategoryFilter>>(filter.value);
-        ret = std::all_of(filters.begin(), filters.end(), [res](const CategoryFilter &f) {
-            return shouldFilter(res, f);
+        ret = std::all_of(filters.begin(), filters.end(), [resource](const CategoryFilter &filter) {
+            return shouldFilter(resource, filter);
         });
         break;
     }
     case CategoryFilter::OrFilter: {
         const auto filters = std::get<QVector<CategoryFilter>>(filter.value);
-        ret = std::any_of(filters.begin(), filters.end(), [res](const CategoryFilter &f) {
-            return shouldFilter(res, f);
+        ret = std::any_of(filters.begin(), filters.end(), [resource](const CategoryFilter &filter) {
+            return shouldFilter(resource, filter);
         });
         break;
     }
     case CategoryFilter::NotFilter: {
         const auto filters = std::get<QVector<CategoryFilter>>(filter.value);
-        ret = !std::any_of(filters.begin(), filters.end(), [res](const CategoryFilter &f) {
-            return shouldFilter(res, f);
+        ret = !std::any_of(filters.begin(), filters.end(), [resource](const CategoryFilter &filter) {
+            return shouldFilter(resource, filter);
         });
         break;
     }
@@ -204,14 +206,14 @@ bool AbstractResource::categoryMatches(Category *cat)
     return shouldFilter(this, cat->filter());
 }
 
-static QSet<Category *> walkCategories(AbstractResource *res, const QVector<Category *> &cats)
+static QSet<Category *> walkCategories(AbstractResource *resource, const QVector<Category *> &categories)
 {
     QSet<Category *> ret;
-    for (Category *cat : cats) {
-        if (res->categoryMatches(cat)) {
-            const auto subcats = walkCategories(res, cat->subCategories());
+    for (auto category : categories) {
+        if (resource->categoryMatches(category)) {
+            const auto subcats = walkCategories(resource, category->subCategories());
             if (subcats.isEmpty()) {
-                ret += cat;
+                ret += category;
             } else {
                 ret += subcats;
             }
@@ -221,9 +223,9 @@ static QSet<Category *> walkCategories(AbstractResource *res, const QVector<Cate
     return ret;
 }
 
-QSet<Category *> AbstractResource::categoryObjects(const QVector<Category *> &cats) const
+QSet<Category *> AbstractResource::categoryObjects(const QVector<Category *> &categories) const
 {
-    return walkCategories(const_cast<AbstractResource *>(this), cats);
+    return walkCategories(const_cast<AbstractResource *>(this), categories);
 }
 
 QUrl AbstractResource::url() const
@@ -244,7 +246,9 @@ QString AbstractResource::executeLabel() const
 
 QString AbstractResource::upgradeText() const
 {
-    QString installed = installedVersion(), available = availableVersion();
+    const auto installed = installedVersion();
+    const auto available = availableVersion();
+
     if (installed == available) {
         // Update of the same version; show when old and new are
         // the same (common with Flatpak runtimes)
@@ -271,15 +275,9 @@ QString AbstractResource::versionString()
     const QString version = isInstalled() ? installedVersion() : availableVersion();
     if (version.isEmpty()) {
         return {};
-    } else {
-        QLocale l;
-        const QString releaseString = l.toString(releaseDate(), QLocale::ShortFormat);
-        if (!releaseString.isEmpty()) {
-            return i18n("%1, released on %2", version, releaseString);
-        } else {
-            return version;
-        }
     }
+
+    return version;
 }
 
 QString AbstractResource::contentRatingDescription() const
@@ -287,17 +285,28 @@ QString AbstractResource::contentRatingDescription() const
     return {};
 }
 
-AbstractResource::ContentIntensity AbstractResource::contentRatingIntensity() const
-{
-    return Mild;
-}
-
-QString AbstractResource::contentRatingText() const
-{
-    return {};
-}
-
 uint AbstractResource::contentRatingMinimumAge() const
 {
     return 0;
 }
+
+QStringList AbstractResource::topObjects() const
+{
+    return {};
+}
+
+QStringList AbstractResource::bottomObjects() const
+{
+    return {};
+}
+
+bool AbstractResource::hasResolvedIcon() const
+{
+    return true;
+}
+
+void AbstractResource::resolveIcon()
+{
+}
+
+#include "moc_AbstractResource.cpp"
