@@ -44,8 +44,6 @@ private:
     QString m_string;
 };
 
-Q_DECLARE_METATYPE(ResourcesCount)
-
 class DISCOVERCOMMON_EXPORT ResourcesProxyModel : public QAbstractListModel, public QQmlParserStatus
 {
     Q_OBJECT
@@ -64,11 +62,11 @@ class DISCOVERCOMMON_EXPORT ResourcesProxyModel : public QAbstractListModel, pub
     Q_PROPERTY(QString extending READ extends WRITE setExtends)
     Q_PROPERTY(bool allBackends READ allBackends WRITE setAllBackends)
     Q_PROPERTY(QVariantList subcategories READ subcategories NOTIFY subcategoriesChanged)
-    Q_PROPERTY(bool isBusy READ isBusy NOTIFY busyChanged)
-    Q_PROPERTY(bool sortByRelevancy READ sortByRelevancy NOTIFY sortByRelevancyChanged)
+    Q_PROPERTY(bool busy READ isBusy NOTIFY busyChanged)
     Q_PROPERTY(ResourcesCount count READ count NOTIFY countChanged FINAL)
 public:
     explicit ResourcesProxyModel(QObject *parent = nullptr);
+    // Make sure the role values don't change since some get stored in config files.
     enum Roles {
         NameRole = Qt::UserRole,
         IconRole,
@@ -91,6 +89,8 @@ public:
         LongDescriptionRole,
         SourceIconRole,
         ReleaseDateRole,
+        // This is better that's always the last value as this one should be never saved to disk
+        SearchRelevanceRole
     };
     Q_ENUM(Roles)
 
@@ -118,7 +118,7 @@ public:
 
     Category *filteredCategory() const;
     QString filteredCategoryName() const;
-    void setFilteredCategoryName(const QString &cat);
+    void setFilteredCategoryName(const QString &categoryName);
 
     QString mimeTypeFilter() const;
     void setMimeTypeFilter(const QString &mime);
@@ -148,13 +148,12 @@ public:
         return m_currentStream != nullptr;
     }
 
-    bool lessThan(AbstractResource *rl, AbstractResource *rr) const;
+    bool orderedLessThan(const StreamResult &left, const StreamResult &right) const;
     Q_SCRIPTABLE void invalidateFilter();
     void invalidateSorting();
 
     bool canFetchMore(const QModelIndex &parent) const override;
     void fetchMore(const QModelIndex &parent) override;
-    bool sortByRelevancy() const;
 
     void classBegin() override
     {
@@ -169,32 +168,33 @@ private Q_SLOTS:
     void removeResource(AbstractResource *resource);
 
 private:
-    void sortedInsertion(const QVector<AbstractResource *> &res);
-    QVariant roleToValue(AbstractResource *res, int role) const;
+    void sortedInsertion(const QVector<StreamResult> &results);
+    QVariant roleToValue(const StreamResult &result, int role) const;
+    QVariant roleToOrderedValue(const StreamResult &result, int role) const;
 
     QVector<int> propertiesToRoles(const QVector<QByteArray> &properties) const;
-    void addResources(const QVector<AbstractResource *> &res);
+    void addResources(const QVector<StreamResult> &results);
     void fetchSubcategories();
-    void removeDuplicates(QVector<AbstractResource *> &newResources);
-    bool isSorted(const QVector<AbstractResource *> &resources);
+    void removeDuplicates(QVector<StreamResult> &newResources);
+    bool isSorted(const QVector<StreamResult> &results);
 
     Roles m_sortRole;
     Qt::SortOrder m_sortOrder;
 
-    bool m_sortByRelevancy;
     bool m_setup = false;
     QString m_categoryName;
 
     AbstractResourcesBackend::Filters m_filters;
     QVariantList m_subcategories;
 
-    QVector<AbstractResource *> m_displayedResources;
+    QVector<StreamResult> m_displayedResources;
     static const QHash<int, QByteArray> s_roles;
+    static QHash<int, int> createRoleToProperty();
     ResultsStream *m_currentStream;
     QTimer m_countTimer;
 
 Q_SIGNALS:
-    void busyChanged(bool isBusy);
+    void busyChanged();
     void sortRoleChanged(int sortRole);
     void sortOrderChanged(Qt::SortOrder order);
     void categoryChanged();
@@ -204,5 +204,4 @@ Q_SIGNALS:
     void resourcesUrlChanged(const QUrl &url);
     void countChanged();
     void filterMinimumStateChanged(bool filterMinimumState);
-    void sortByRelevancyChanged(bool sortByRelevancy);
 };
