@@ -11,45 +11,61 @@
 #include <KNotificationJobUiDelegate>
 #include <KPluginFactory>
 #include <QDBusConnection>
+#include <QDBusConnectionInterface>
 #include <QDBusMessage>
 #include <QDBusPendingCall>
 #include <QDebug>
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-#include <QNetworkConfigurationManager>
-#else
 #include <QNetworkInformation>
-#endif
 #include <QProcess>
 
 #include <KIO/ApplicationLauncherJob>
 #include <KIO/CommandLauncherJob>
 
 #include "../libdiscover/utils.h"
+#include "../libdiscover/UpdateModel/RefreshNotifierDBus.h"
+#include "Login1ManagerInterface.h"
 #include "updatessettings.h"
 #include <chrono>
 
+#include "debug.h"
+
 using namespace std::chrono_literals;
 
-DiscoverNotifier::DiscoverNotifier(QObject *parent)
+namespace
+{
+bool isOnline()
+{
+    return !QNetworkInformation::instance() || QNetworkInformation::instance()->reachability() == QNetworkInformation::Reachability::Online;
+}
+} // namespace
+
+DiscoverNotifier::DiscoverNotifier(const std::chrono::seconds &checkDelay, QObject *parent)
     : QObject(parent)
 {
     m_settings = new UpdatesSettings(this);
     m_settingsWatcher = KConfigWatcher::create(m_settings->sharedConfig());
-#if QT_VERSION >= QT_VERSION_CHECK(6, 3, 0)
-    QNetworkInformation::instance()->load(QNetworkInformation::Feature::Reachability | QNetworkInformation::Feature::TransportMedium);
-    connect(QNetworkInformation::instance(), &QNetworkInformation::reachabilityChanged, this, &DiscoverNotifier::stateChanged);
-    connect(QNetworkInformation::instance(), &QNetworkInformation::transportMediumChanged, this, &DiscoverNotifier::stateChanged);
-#endif
+    QNetworkInformation::loadBackendByFeatures(QNetworkInformation::Feature::Reachability | QNetworkInformation::Feature::TransportMedium);
+    if (auto info = QNetworkInformation::instance()) {
+        connect(info, &QNetworkInformation::reachabilityChanged, this, &DiscoverNotifier::stateChanged);
+        connect(info, &QNetworkInformation::transportMediumChanged, this, &DiscoverNotifier::stateChanged);
+        connect(info, &QNetworkInformation::isBehindCaptivePortalChanged, this, &DiscoverNotifier::stateChanged);
+    } else {
+        qWarning() << "QNetworkInformation has no backend. Is NetworkManager.service running?";
+    }
 
     refreshUnattended();
     connect(m_settingsWatcher.data(), &KConfigWatcher::configChanged, this, [this](const KConfigGroup &group, const QByteArrayList &names) {
-        if (group.config()->name() == m_settings->config()->name() && group.name() == "Global" && names.contains("UseUnattendedUpdates")) {
+        if (group.config()->name() != m_settings->config()->name() || group.name() != QLatin1String("Global")) {
+            return;
+        }
+        if (names.contains("UseUnattendedUpdates") || names.contains("RequiredNotificationInterval")) {
             refreshUnattended();
+            Q_EMIT stateChanged();
         }
     });
 
     m_backends = BackendNotifierFactory().allBackends();
-    for (BackendNotifierModule *module : qAsConst(m_backends)) {
+    for (BackendNotifierModule *module : std::as_const(m_backends)) {
         connect(module, &BackendNotifierModule::foundUpdates, this, &DiscoverNotifier::updateStatusNotifier);
         connect(module, &BackendNotifierModule::needsRebootChanged, this, [this]() {
             // If we are using offline updates, there is no need to badger the user to
@@ -70,7 +86,26 @@ DiscoverNotifier::DiscoverNotifier(QObject *parent)
     updateStatusNotifier();
 
     // Only fetch updates after the system is comfortably booted
-    QTimer::singleShot(0s, this, &DiscoverNotifier::recheckSystemUpdateNeeded);
+    QTimer::singleShot(checkDelay, this, &DiscoverNotifier::recheckSystemUpdateNeeded);
+
+    auto login1 = new OrgFreedesktopLogin1ManagerInterface(QStringLiteral("org.freedesktop.login1"),
+                                                           QStringLiteral("/org/freedesktop/login1"),
+                                                           QDBusConnection::systemBus(),
+                                                           this);
+    connect(login1, &OrgFreedesktopLogin1ManagerInterface::PrepareForSleep, this, [this, checkDelay](bool sleeping) {
+        // if we just woke up and have not checked in the notification interval
+        if (!sleeping && m_lastUpdate.secsTo(QDateTime::currentDateTimeUtc()) > m_settings->requiredNotificationInterval()) {
+            QTimer::singleShot(checkDelay, this, &DiscoverNotifier::recheckSystemUpdateNeeded);
+        }
+    });
+
+    // Listen to broadcasts from discover about notification changes.
+    QDBusConnection::sessionBus().connect(QString(),
+                                          RefreshNotifierDBus::path,
+                                          RefreshNotifierDBus::interface,
+                                          RefreshNotifierDBus::notifyNotifier,
+                                          this,
+                                          SLOT(recheckSystemUpdateNeeded()));
 }
 
 DiscoverNotifier::~DiscoverNotifier() = default;
@@ -104,10 +139,12 @@ bool DiscoverNotifier::notifyAboutUpdates() const
 {
     if (state() != NormalUpdates && state() != SecurityUpdates) {
         // it's not very helpful to notify that everything is in order
+        qCDebug(NOTIFIER) << "Not notifying about updates, state is" << state();
         return false;
     }
 
     if (m_settings->requiredNotificationInterval() < 0) {
+        qCDebug(NOTIFIER) << "Not notifying about updates, requiredNotificationInterval is" << m_settings->requiredNotificationInterval();
         return false;
     }
 
@@ -115,19 +152,15 @@ bool DiscoverNotifier::notifyAboutUpdates() const
     // kwriteconfig5 --file PlasmaDiscoverUpdates --group Global --key RequiredNotificationInterval 3600
     const QDateTime earliestNextNotificationTime = m_settings->lastNotificationTime().addSecs(m_settings->requiredNotificationInterval());
     if (earliestNextNotificationTime.isValid() && earliestNextNotificationTime > QDateTime::currentDateTimeUtc()) {
+        qCDebug(NOTIFIER) << "Not notifying about updates, earliestNextNotificationTime is" << earliestNextNotificationTime;
         return false;
     }
 
     m_settings->setLastNotificationTime(QDateTime::currentDateTimeUtc());
     m_settings->save();
 
-    auto method = QDBusMessage::createMethodCall(QStringLiteral("org.kde.discover"),
-                                                 QStringLiteral("/"),
-                                                 QStringLiteral("org.freedesktop.DBus.Peer"),
-                                                 QStringLiteral("Ping"));
-    auto call = QDBusConnection::sessionBus().asyncCall(method);
-    call.waitForFinished();
-    if (call.isValid()) {
+    if (QDBusConnection::sessionBus().interface()->isServiceRegistered(QStringLiteral("org.kde.discover"))) {
+        qCDebug(NOTIFIER) << "Not notifying about updates, discover is running";
         return false;
     }
     return true;
@@ -140,23 +173,25 @@ void DiscoverNotifier::showUpdatesNotification()
     }
 
     if (!notifyAboutUpdates()) {
+        qCDebug(NOTIFIER) << "showUpdatesNotification: not notifying about updates";
         return;
     }
+    qCDebug(NOTIFIER) << "showUpdatesNotification: notifying about updates";
 
-    m_updatesAvailableNotification = KNotification::event(QStringLiteral("Update"),
-                                                          message(),
-                                                          {},
-                                                          iconName(),
-                                                          nullptr,
-                                                          KNotification::CloseOnTimeout,
-                                                          QStringLiteral("discoverabstractnotifier"));
+    m_updatesAvailableNotification =
+        KNotification::event(QStringLiteral("Update"), message(), {}, iconName(), KNotification::CloseOnTimeout, QStringLiteral("discoverabstractnotifier"));
     m_updatesAvailableNotification->setHint(QStringLiteral("resident"), true);
     const QString name = i18n("View Updates");
-    m_updatesAvailableNotification->setDefaultAction(name);
-    m_updatesAvailableNotification->setActions({name});
-    connect(m_updatesAvailableNotification, QOverload<unsigned int>::of(&KNotification::activated), this, [this] {
+
+    auto showUpdates = [this] {
         showDiscoverUpdates(m_updatesAvailableNotification->xdgActivationToken());
-    });
+    };
+
+    auto defaultAction = m_updatesAvailableNotification->addDefaultAction(name);
+    connect(defaultAction, &KNotificationAction::activated, this, showUpdates);
+
+    auto showUpdatesAction = m_updatesAvailableNotification->addAction(name);
+    connect(showUpdatesAction, &KNotificationAction::activated, this, showUpdates);
 }
 
 void DiscoverNotifier::updateStatusNotifier()
@@ -168,6 +203,8 @@ void DiscoverNotifier::updateStatusNotifier()
                                 return module->hasUpdates();
                             });
 
+    qCDebug(NOTIFIER) << "updateStatusNotifier: hasUpdates" << hasUpdates << "hasSecurityUpdates" << hasSecurityUpdates;
+
     if (m_hasUpdates == hasUpdates && m_hasSecurityUpdates == hasSecurityUpdates)
         return;
 
@@ -182,15 +219,15 @@ void DiscoverNotifier::updateStatusNotifier()
 }
 
 // we only want to do unattended updates when on an ethernet or wlan network
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-static bool isConnectionAdequate(const QNetworkConfiguration &network)
-{
-    return (network.bearerType() == QNetworkConfiguration::BearerEthernet || network.bearerType() == QNetworkConfiguration::BearerWLAN);
-}
-#elif QT_VERSION >= QT_VERSION_CHECK(6, 3, 0)
 static bool isConnectionAdequate()
 {
     const auto info = QNetworkInformation::instance();
+    if (!info) {
+        return false; // no backend available (e.g. NetworkManager not running), assume not adequate
+    }
+    if (info->supports(QNetworkInformation::Feature::CaptivePortal) && info->isBehindCaptivePortal()) {
+        return false;
+    }
     if (info->supports(QNetworkInformation::Feature::Metered)) {
         return !info->isMetered();
     } else {
@@ -198,28 +235,27 @@ static bool isConnectionAdequate()
         return transport == QNetworkInformation::TransportMedium::Ethernet || transport == QNetworkInformation::TransportMedium::WiFi;
     }
 }
-#endif
 
 void DiscoverNotifier::refreshUnattended()
 {
     m_settings->read();
 
     if (!notifyAboutUpdates()) {
+        qCDebug(NOTIFIER) << "refreshUnattended: not notifying about updates";
         return;
     }
 
-    const auto enabled = m_settings->useUnattendedUpdates()
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-        && m_manager->isOnline() && isConnectionAdequate(m_manager->defaultConfiguration());
-#elif QT_VERSION >= QT_VERSION_CHECK(6, 3, 0)
-        && QNetworkInformation::instance()->reachability() == QNetworkInformation::Reachability::Online && isConnectionAdequate();
-#endif
+    qCDebug(NOTIFIER) << "refreshUnattended: useUnattendedUpdates" << m_settings->useUnattendedUpdates() << "isOnline" << isOnline() << "isConnectionAdequate"
+                      << isConnectionAdequate();
+    const auto enabled = m_settings->useUnattendedUpdates() && isOnline() && isConnectionAdequate();
     if (bool(m_unattended) == enabled)
         return;
 
     if (enabled) {
+        qCDebug(NOTIFIER) << "Enabling unattended updates";
         m_unattended = new UnattendedUpdates(this);
     } else {
+        qCDebug(NOTIFIER) << "Disabling unattended updates";
         delete m_unattended;
         m_unattended = nullptr;
     }
@@ -231,11 +267,7 @@ DiscoverNotifier::State DiscoverNotifier::state() const
         return RebootRequired;
     else if (m_isBusy)
         return Busy;
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    else if (m_manager && !m_manager->isOnline())
-#else
-    else if (QNetworkInformation::instance()->reachability() != QNetworkInformation::Reachability::Online)
-#endif
+    else if (!isOnline())
         return Offline;
     else if (m_hasSecurityUpdates)
         return SecurityUpdates;
@@ -255,7 +287,7 @@ QString DiscoverNotifier::iconName() const
     case NoUpdates:
         return QStringLiteral("update-none");
     case RebootRequired:
-        return QStringLiteral("system-reboot");
+        return QStringLiteral("system-reboot-update");
     case Offline:
         return QStringLiteral("offline");
     case Busy:
@@ -285,17 +317,8 @@ QString DiscoverNotifier::message() const
 
 void DiscoverNotifier::recheckSystemUpdateNeeded()
 {
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    if (!m_manager) {
-        m_manager = new QNetworkConfigurationManager(this);
-        connect(m_manager, &QNetworkConfigurationManager::onlineStateChanged, this, &DiscoverNotifier::stateChanged);
-        if (!m_manager->isOnline()) {
-            Q_EMIT stateChanged();
-        }
-    }
-#endif
-
-    for (BackendNotifierModule *module : qAsConst(m_backends))
+    m_lastUpdate = QDateTime::currentDateTimeUtc();
+    for (BackendNotifierModule *module : std::as_const(m_backends))
         module->recheckSystemUpdateNeeded();
 
     refreshUnattended();
@@ -315,18 +338,18 @@ void DiscoverNotifier::showRebootNotification()
                                                        i18n("Restart is required"),
                                                        i18n("The system needs to be restarted for the updates to take effect."),
                                                        QStringLiteral("system-software-update"),
-                                                       nullptr,
                                                        KNotification::Persistent | KNotification::DefaultEvent,
                                                        QStringLiteral("discoverabstractnotifier"));
 
-    notification->setActions(QStringList{i18nc("@action:button", "Restart")});
-    notification->setDefaultAction(notification->actions().constFirst());
-    connect(notification, &KNotification::action1Activated, this, &DiscoverNotifier::reboot);
+    auto restartAction = notification->addAction(i18nc("@action:button", "Update and Restart"));
+    auto shutdownAction = notification->addAction(i18nc("@action:button", "Update and Shut Down"));
+    connect(restartAction, &KNotificationAction::activated, this, &DiscoverNotifier::rebootPrompt);
+    connect(shutdownAction, &KNotificationAction::activated, this, &DiscoverNotifier::shutdownPrompt);
 
     notification->sendEvent();
 }
 
-void DiscoverNotifier::reboot()
+void DiscoverNotifier::rebootPrompt()
 {
     auto method = QDBusMessage::createMethodCall(QStringLiteral("org.kde.LogoutPrompt"),
                                                  QStringLiteral("/LogoutPrompt"),
@@ -335,18 +358,47 @@ void DiscoverNotifier::reboot()
     QDBusConnection::sessionBus().asyncCall(method);
 }
 
+void DiscoverNotifier::shutdownPrompt()
+{
+    auto method = QDBusMessage::createMethodCall(QStringLiteral("org.kde.LogoutPrompt"),
+                                                 QStringLiteral("/LogoutPrompt"),
+                                                 QStringLiteral("org.kde.LogoutPrompt"),
+                                                 QStringLiteral("promptShutDown"));
+    QDBusConnection::sessionBus().asyncCall(method);
+}
+
+void DiscoverNotifier::promptAll()
+{
+    auto method = QDBusMessage::createMethodCall(QStringLiteral("org.kde.LogoutPrompt"),
+                                                 QStringLiteral("/LogoutPrompt"),
+                                                 QStringLiteral("org.kde.LogoutPrompt"),
+                                                 QStringLiteral("promptAll"));
+    QDBusConnection::sessionBus().asyncCall(method);
+}
+
 void DiscoverNotifier::foundUpgradeAction(UpgradeAction *action)
 {
-    KNotification *notification = new KNotification(QStringLiteral("distupgrade-notification"), KNotification::Persistent | KNotification::DefaultEvent);
-    notification->setIconName(QStringLiteral("system-software-update"));
-    notification->setActions(QStringList{i18nc("@action:button", "Upgrade")});
-    notification->setTitle(i18n("Upgrade available"));
-    notification->setText(i18n("New version: %1", action->description()));
+    updateStatusNotifier();
 
-    connect(notification, &KNotification::action1Activated, this, [action]() {
+    if (!notifyAboutUpdates()) {
+        return;
+    }
+
+    KNotification *notification = new KNotification(QStringLiteral("DistUpgrade"), KNotification::Persistent);
+    notification->setIconName(QStringLiteral("system-software-update"));
+    notification->setTitle(i18n("Upgrade available"));
+    notification->setText(i18nc("A new distro release (name and version) is available for upgrade", "%1 is now available.", action->description()));
+    notification->setComponentName(QStringLiteral("discoverabstractnotifier"));
+
+    auto upgradeAction = notification->addAction(i18nc("@action:button", "Upgrade"));
+    connect(upgradeAction, &KNotificationAction::activated, this, [action] {
         action->trigger();
     });
 
+    connect(action, &UpgradeAction::showDiscoverUpdates, this, [this, notification]() {
+        showDiscoverUpdates(notification->xdgActivationToken());
+    });
+
     notification->sendEvent();
 }
 
@@ -356,6 +408,14 @@ void DiscoverNotifier::setBusy(bool isBusy)
         return;
 
     m_isBusy = isBusy;
-    Q_EMIT busyChanged(isBusy);
+    Q_EMIT busyChanged();
     Q_EMIT stateChanged();
 }
+
+void DiscoverNotifier::recheckSystemUpdateNeededAndNotifyApp()
+{
+    recheckSystemUpdateNeeded();
+    RefreshNotifierDBus::notify(RefreshNotifierDBus::notifyApp);
+}
+
+#include "moc_DiscoverNotifier.cpp"
